<?php
/**
 * @file
 * 'Wrapper: Element' style.
 * @todo Container markup and attributes.
 * @todo Multiple attribute name and value pairs with ajaxy 'Add more' button.
 */

// Plugin definition.
$plugin = array(
  'title' => t('Wrapper: Element'),
  'description' => t('Wrap regions and panes with HTML elements.'),
  'render region' => 'panels_extra_styles_wrapper_element_render',
  'render pane' => 'panels_extra_styles_wrapper_element_render',
  'settings form' => 'panels_extra_styles_wrapper_element_region_settings_form',
  'pane settings form' => 'panels_extra_styles_wrapper_element_pane_settings_form',
);

/**
 * Render callback.
 */
function theme_panels_extra_styles_wrapper_element_render($vars) {
  $settings = $vars['settings'];

  $title = !empty($vars['content']->title) ? $vars['content']->title : NULL;
  $content = NULL;
  // Region or pane?
  if (!empty($vars['panes'])) {
    $content = render(implode(PHP_EOL, $vars['panes']));
  }
  else {
    $content = render($vars['content']->content);
  }

  // Wrap it!
  if (!empty($title) && !empty($settings['title']['element']) && $settings['title']['element'] != 'no_wrapper') {
    $title_id = !empty($settings['title']['attributes']['id']) ? ' id="#' . $settings['title']['attributes']['id'] . '"' : NULL;
    $title_class = !empty($settings['title']['attributes']['class']) ? ' class="' . $settings['title']['attributes']['class'] . '"' : NULL;
    $title = '<' . $settings['title']['element'] . $title_id . $title_class . '>' . $title . '</' . $settings['title']['element'] . '>';
  }
  if (!empty($content) && !empty($settings['content']['element']) && $settings['content']['element'] != 'no_wrapper') {
    $content_id = !empty($settings['content']['attributes']['id']) ? ' id="#' . $settings['content']['attributes']['id'] . '"' : NULL;
    $content_class = !empty($settings['content']['attributes']['class']) ? ' class="' . $settings['content']['attributes']['class'] . '"' : NULL;
    $content = '<' . $settings['content']['element'] . $content_id . $content_class . '>' . $content . '</' . $settings['content']['element'] . '>';
  }

  $output = PHP_EOL . $title . PHP_EOL . $content . PHP_EOL;

  return $output;
}

/**
 * Region settings form callback.
 */
function panels_extra_styles_wrapper_element_region_settings_form($style_settings) {
  // Content.
  $element_default_value = isset($style_settings['content']['element']) ? $style_settings['content']['element'] : '';
  $form['content']['element'] = array(
    '#title' => t('Content'),
    '#description' => t('HTML element to wrap around the content.'),
  ) + _panels_extra_styles_wrapper_element_options($element_default_value);

  // Content attributes.
  $attributes_default_value = isset($style_settings['content']['attributes']) ? $style_settings['content']['attributes'] : array();
  $form['content']['attributes'] = array(
    '#title' => t('Content Attributes'),
  ) + _panels_extra_styles_wrapper_attribute_inputs($attributes_default_value);

  return $form;
}

/**
 * Pane settings form callback.
 */
function panels_extra_styles_wrapper_element_pane_settings_form($style_settings) {
  // Title.
  $element_default_value = isset($style_settings['title']['element']) ? $style_settings['title']['element'] : '';
  $form['title']['element'] = array(
    '#title' => t('Title'),
    '#description' => t('HTML element to wrap around the title.'),
  ) + _panels_extra_styles_wrapper_element_options($element_default_value);

  // Title attributes.
  $attributes_default_value = isset($style_settings['title']['attributes']) ? $style_settings['title']['attributes'] : array();
  $form['title']['attributes'] = array(
    '#title' => t('Title Attributes'),
  ) + _panels_extra_styles_wrapper_attribute_inputs($attributes_default_value);

  // Content form items, grabbed from the region settings.
  $form += panels_extra_styles_wrapper_element_region_settings_form($style_settings);

  return $form;
}

/**
 * Element options for settings forms.
 */
function _panels_extra_styles_wrapper_element_options($default_value = '') {
  $options = array(
    '#type' => 'select',
    '#default_value' => $default_value,
  );

  if (module_exists('fences') && function_exists('fences_get_fences_options')) {
    // Element options, grabbed from the fences module.
    $options['#options'] = fences_get_fences_options('field');
  }
  else {
    // Notify user of fences awesomenss.
    drupal_set_message(t('Install the <a href="@fences_link">fences</a> module for a <em>plethora</em> of options!',
      array('@fences_link' => url('http://drupal.org/project/fences'))),
      'warning', FALSE);

    // Fallback element options, grabbed from View's defaults.
    $options['#options'] = variable_get('views_field_rewrite_elements', array(
      '' => t('- Use default -'),
      '0' => t('- None -'),
      'div' => 'DIV',
      'span' => 'SPAN',
      'h1' => 'H1',
      'h2' => 'H2',
      'h3' => 'H3',
      'h4' => 'H4',
      'h5' => 'H5',
      'h6' => 'H6',
      'p' => 'P',
      'strong' => 'STRONG',
      'em' => 'EM',
    ));
  }

  return $options;
}

/**
 * Attribute inputs for settings forms.
 */
function _panels_extra_styles_wrapper_attribute_inputs($default_value = array()) {
  $inputs = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $inputs['id'] = array(
    '#type' => 'textfield',
    '#title' => t('id'),
    '#description' => t('CSS id to apply to the element, without the hash.'),
    '#default_value' => isset($default_value['id']) ? $default_value['id'] : '',
  );
  $inputs['class'] = array(
    '#type' => 'textfield',
    '#title' => t('class'),
    '#description' => t('CSS classes to apply to the element, separated by spaces.'),
    '#default_value' => isset($default_value['class']) ? $default_value['class'] : '',
  );

  return $inputs;
}
