<?php
/**
 * @file
 * 'Wrapper: Raw' style.
 */

// Plugin definition.
$plugin = array(
  'title' => t('Wrapper: Raw'),
  'description' => t('Wrap regions and panes with raw HTML.'),
  'render region' => 'panels_extra_styles_wrapper_raw_render',
  'render pane' => 'panels_extra_styles_wrapper_raw_render',
  'settings form' => 'panels_extra_styles_wrapper_raw_region_settings_form',
  'pane settings form' => 'panels_extra_styles_wrapper_raw_pane_settings_form',
);

/**
 * Render callback.
 */
function theme_panels_extra_styles_wrapper_raw_render($vars) {
  $settings = $vars['settings'];

  $title = !empty($vars['content']->title) ? $vars['content']->title : NULL;
  $content = NULL;
  // Region or pane?
  if (!empty($vars['panes'])) {
    $content = render(implode(PHP_EOL, $vars['panes']));
  }
  else {
    $content = render($vars['content']->content);
  }

  // Wrap it!
  if (!empty($title)) {
    $title_prefix = isset($settings['title']['prefix']) ? $settings['title']['prefix'] : NULL;
    $title_suffix = isset($settings['title']['suffix']) ? $settings['title']['suffix'] : NULL;
    $title = $title_prefix . $title . $title_suffix;
  }
  if (!empty($content)) {
    $content_prefix = isset($settings['content']['prefix']) ? $settings['content']['prefix'] : NULL;
    $content_suffix = isset($settings['content']['suffix']) ? $settings['content']['suffix'] : NULL;
    $content = $content_prefix . $content . $content_suffix;
  }

  $output = PHP_EOL . $title . PHP_EOL . $content . PHP_EOL;

  return $output;
}

/**
 * Region settings form callback.
 */
function panels_extra_styles_wrapper_raw_region_settings_form($style_settings) {
  $default_settings = isset($style_settings['content']) ? $style_settings['content'] : array();
  $form['content'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content'),
  ) + _panels_extra_styles_wrapper_raw_prefix_suffix_inputs($default_settings);

  return $form;
}

/**
 * Pane settings form callback.
 */
function panels_extra_styles_wrapper_raw_pane_settings_form($style_settings) {
  $default_settings = isset($style_settings['title']) ? $style_settings['title'] : array();
  $form['title'] = array(
    '#type' => 'fieldset',
    '#title' => t('Title'),
  ) + _panels_extra_styles_wrapper_raw_prefix_suffix_inputs($default_settings);

  // Content form items, grabbed from region settings.
  $form += panels_extra_styles_wrapper_raw_region_settings_form($style_settings);

  return $form;
}

/**
 * Prefix & Suffix inputs for settings form.
 */
function _panels_extra_styles_wrapper_raw_prefix_suffix_inputs($default_value = array()) {
  $inputs['prefix'] = array(
    '#type' => 'textarea',
    '#title' => t('Prefix'),
    '#description' => t('HTML code to go <em>before</em> the content.'),
    '#default_value' => isset($default_value['prefix']) ? $default_value['prefix'] : '',
  );
  $inputs['suffix'] = array(
    '#type' => 'textarea',
    '#title' => t('Suffix'),
    '#description' => t('HTML code to go <em>after</em> the content.'),
    '#default_value' => isset($default_value['suffix']) ? $default_value['suffix'] : '',
  );

  return $inputs;
}
